#! /usr/bin/python
# Copyright 2015,2016,2018 Cumulus Networks Inc.  All rights reserved

#############################################################################
# The basic POE class. Platform specific and chip specific implementations
# inherit from this class. It is also the default for platforms that do not
# support POE.
#


try:
    import exceptions
    import sys
    import os
    import StringIO
    import ConfigParser
    import cPickle
    import fcntl
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

POE_VERBOSITY_QUIET=1
POE_VERBOSITY_VERBOSE=2
POE_VERBOSITY_DEBUG=3

PORT_CFG_PRIORITY_LOW=0
PORT_CFG_PRIORITY_HIGH=1
PORT_CFG_PRIORITY_CRITICAL=2

MAX_POE_POWER=15000
MAX_POEPLUS_POWER=25500
MAX_UPOE_POWER=60000

class POEDRuntimeError(RuntimeError):
    pass

class POEDSequenceError(RuntimeError):
    pass

def poe():
    return PoeBase()

class PoeBase():
    persistent_cfg_path='/var/cache/cumulus'      # indicates first-ever boot
    persistent_cfg_node='poe'
    volatile_cfg_path='/run/cumulus/poe'      # indicates warm/cold boot
    volatile_cfg_node='boot_key'
    cfg_key = 'POE'
    class_based_power_map = {
        '0':16000, '1':5000, '2':8000, '3':17000, '4':32000, '5':40000
    }

    def __init__(self, verbosity=POE_VERBOSITY_QUIET):
        self.log = PoeLogger(verbosity)
        self.is_first_boot = self._check_first_boot()
        self.is_warm_cold_boot = self._check_warm_cold_boot()

    def get_poe_params(self):
        return PoeParams()

    def poe_enable(self):
        ''' Most platforms require POE to be enabled in the CPLD before anything
            will work. This is the place to do it.
        '''
        pass

    def poe_disable(self):
        ''' When the poed exits, we want to globally disable POE. This routine
            should do that.
        '''
        pass

    def handle_first_boot(self):
        ''' This gets executed on a 'first ever' boot. The place to do
            any non-volatile configuration.
        '''
        pass

    def handle_warm_cold_boot(self):
        ''' This gets executed the first time poed starts after a warm or cold
            boot.
        '''
        self.load_saved_configuration()

    def handle_poed_start(self):
        ''' This gets executed every time poed starts
        '''
        pass

    def get_running_state(self):
        return PoeStatus().load_running_state()

    def save_running_state(self, system_state):
        PoeStatus().save_running_state(system_state)

    def adjust_system_poe_power(self, prev_index=0):
        raise NotImplementedError

    def get_poe_system_status(self, incremental=False):
        raise NotImplementedError

    def get_port_extra_status(self, portList):
        '''
        This is used to get the info for a port that does not
        get populated by get_system_status call.

        :param portList: the list of ports to get extra status for
        :return: a list of PortExtraStatusResults, one for each port
        '''
        pass

    def get_port_4pair_state(self, portList):
        return NotImplementedError

    def get_poe_version_info(self):
        raise NotImplementedError

    def get_poe_diag_info(self):
        raise NotImplementedError

    def get_non_lldp_allocated_power(self, pd_class, power):
        return self.class_based_power_map.get(pd_class, float(power)*1000)

    ''' set commands either succeed quietly or raise an exception
    '''
    def set_port_enable_poe(self, portList):
        raise NotImplementedError

    def set_port_disable_poe(self, portList):
        raise NotImplementedError

    def reset_ports(self, portList):
        ''' we 'reset' a port by turning off, waiting a short period of time and then turning
            it back on. This resets the device that is connected to the port.
        '''
        pass

    def set_port_priority(self, portList, level):
        raise NotImplementedError

    def set_enable_4pair_power(self, portList):
        raise NotImplementedError

    def set_disable_4pair_power(self, portList):
        pass

    def set_port_l2_power(self, portList, allocated, requested):
        raise NotImplementedError

    def exit_port_l2_mode(self, portList):
        '''
        notify the POE hardware that the port is no longer in L2 mode.
        not necessary on all chips
        '''
        pass

    def save_running_configuration(self):
        PoeConfig(self).save_running_config()

    def load_saved_configuration(self):
        PoeConfig(self).load_saved_config()
        PoeStatus().save_running_state(self.get_poe_system_status())

    def save_sensors_info(self, poe_params):
        PoeStatus().save_sensors_info(poe_params)

    def save_sensors_data(self, system_status, poe_params):
        PoeStatus().save_sensors_data(system_status, poe_params)

    def log_verbose(self, msg):
        self.log.verbose('VRB- %s' % msg)

    def log_debug(self, msg):
        self.log.debug('DBG- %s' % msg)

    def _check_first_boot(self):
        is_first = self._get_cfg_key(self.persistent_cfg_path,
                                     self.persistent_cfg_node) != self.cfg_key
        if is_first:
            self._update_cfg_key(self.persistent_cfg_path,
                                 self.persistent_cfg_node, self.cfg_key)
        return is_first

    def _check_warm_cold_boot(self):
        if self.is_first_boot or self._get_cfg_key(self.volatile_cfg_path,
                                                   self.volatile_cfg_node) != self.cfg_key:
            self._update_cfg_key(self.volatile_cfg_path,
                                 self.volatile_cfg_node, self.cfg_key)
            return True
        return False

    def _update_cfg_key(self, path, node, key):
        try:
            if not os.path.exists(path):
                os.makedirs(path)
            with open(os.path.join(path, node), 'w') as fp:
                fp.write(key)
        except:
            return False
        return True

    def _get_cfg_key(self, path, node):
        try:
            key = open(os.path.join(path, node)).read().rsplit('\n')[0]
        except:
            key = None
        return key

    def _bytes_to_str(self, bytes):
        respStr = ''
        spaceStr = ''
        for i in range(len(bytes)):
            respStr += '%s0x%02X' % (spaceStr, bytes[i])
            spaceStr = ' '
        return respStr

    def _mk_list(self, val):
        if not hasattr(val, '__iter__'):
            val = [val]
        return val

    def __str__(self):
        return 'poe_base class'

class PoeSensorParams(object):
    def __init__(self, name, sensor_id, max, crit):
        self.name = name
        self.sensor_id = sensor_id
        self.max = max
        self.crit = crit

class PoePortParams(object):
    def __init__(self, port_id, swp, poe_supported, base_power, support_4pair, max_power):
        self.port_id = port_id
        self.swp = swp
        self.poe_supported = poe_supported
        self.base_power = base_power
        self.support_4pair = support_4pair
        self.max_power = max_power

    def __str__(self):
        return '  %u: %s, %s, %u, %s, %u\n' % (self.port_id, self.swp, str(self.poe_supported), self.base_power, str(self.support_4pair), self.max_power)

class PoeParams():
    def __init__(self, supported=False, interval=60, ports=[], sensors={}):
        self.supported = supported
        self.interval = interval
        self.ports = ports
        self.sensors = sensors

    def __str__(self):
        outstr = '%s, %u\n' % (str(self.supported), self.interval)
        for id, port in self.ports.items():
            outstr += str(port)

class PoeLogger():
    def __init__(self, level=POE_VERBOSITY_QUIET):
        self.level = level

    def set_level(self, level):
        self.level = level

    def verbose(self, msg):
        self._log(POE_VERBOSITY_VERBOSE, msg)

    def verbose(self, msg):
        self._log(POE_VERBOSITY_VERBOSE, msg)

    def debug(self, msg):
        self._log(POE_VERBOSITY_DEBUG, msg)

    def _log(self, level, msg):
        if level <= self.level:
            print msg

class PoeStatus():
    state_dir='/run/cache/cumulus/unit_state/POE'
    running_state_name='poe_running_dump'
    sensors_dir=os.path.join(PoeBase.volatile_cfg_path, 'sensors')

    def save_running_state(self, system_state):
        try:
            if not os.path.exists(self.state_dir):
                os.makedirs(self.state_dir)
            path = os.path.join(self.state_dir, self.running_state_name)
            f = open(path, 'w')
            fcntl.flock(f, fcntl.LOCK_EX)
            cPickle.dump(system_state, f)
            fcntl.flock(f, fcntl.LOCK_UN)
            f.close()
        except IOError, e:
            PoeLogger().verbose("Exception: %s" % str(e))
            # smond probably hasn't started yet
            pass

    def save_sensors_info(self, poe_params):
        for sensor_id, sensor in poe_params.sensors.items():
            self._write_sensor_file('%s_max' % sensor.name, sensor.max)
            self._write_sensor_file('%s_crit' % sensor.name, sensor.crit)

    def save_sensors_data(self, system_state, poe_params):
        for sensor_id, sensor in poe_params.sensors.items():
            data = system_state.get('sensors', {}).get(sensor_id)
            if data != None:
                self._write_sensor_file('%s_input' % sensor.name, data)

    def _write_sensor_file(self, file_name, data):
        try:
            if not os.path.exists(self.sensors_dir):
                os.makedirs(self.sensors_dir)
            path = os.path.join(self.sensors_dir, file_name)
            f = open(path, 'w')
            fcntl.flock(f, fcntl.LOCK_EX)
            f.write('%s\n' % str(data))
            fcntl.flock(f, fcntl.LOCK_UN)
            f.close
        except IOError, e:
            # smond probably hasn't started yet
            pass

    def load_running_state(self):
        try:
            path = os.path.join(self.state_dir, self.running_state_name)
            f = open(path)
            fcntl.flock(f, fcntl.LOCK_EX)
            system_state = cPickle.load(f)
            fcntl.flock(f, fcntl.LOCK_UN)
            f.close()
        except:
            return None
        return system_state

class PoeConfig():
    config_path_file='/etc/cumulus/poe.conf'
    cache_config_path='/run/cache/cumulus/unit_status/POE'
    cache_config_name='running_config'

    def __init__(self, poe):
        self.poe = poe

    def save_running_config(self):
        try:
            if not os.path.exists(self.cache_config_path):
                os.makedirs(self.cache_config_path)
            f = open(self.config_path_file, 'w')
            config = ConfigParser.ConfigParser()
            config.add_section('enable')
            config.add_section('priority')
            all_port_status = self.poe.get_running_state().get('ports')
            extra_port_status = self.poe.get_port_extra_status(self.poe.get_poe_params().ports)
            for port_num in all_port_status:
                port_status = all_port_status[port_num]
                extra_status = extra_port_status[port_num]
                status = 'enable'
                if port_status.get('hw_status') == 'disabled':
                    status = 'disable'
                config.set('enable', 'swp%u' % (port_num + 1), status)
                config.set('priority', 'swp%u' % (port_num + 1), extra_status.get('priority', 'low'))
            config.write(f)
            f.close()
        except Exception, e:
            print 'Error writing to config file "%s".' % (self.config_path_file)
            print 'Exception: %s' % (str(e))
            return

    def load_saved_config(self):
        '''
        loads a saved configuration and applies it. Will apply defaults to anything
        that is missing (or if there is no config file at all).
        '''
        try:
            configFP = StringIO.StringIO(open(self.config_path_file, 'r').read())
            parser = ConfigParser.RawConfigParser()
            parser.readfp(configFP)
        except:
            pass

        sys_status = self.poe.get_poe_system_status(True)
        try:
            enable_ports = dict(parser.items('enable'))
        except:
            enable_ports = {}
        try:
            priority_ports = dict(parser.items('priority'))
        except:
            priority_ports = {}

        set_enable_ports = {}
        set_priority_ports = {}
        for port in self.poe.get_poe_params().ports:
            swp = 'swp%u' % (port + 1)
            set_enable_ports[port] = enable_ports.get(swp, 'enable')
            set_priority_ports[port] = self.parse_priority(swp, priority_ports.get(swp, 'low'))

        for port in set_enable_ports:
            val = set_enable_ports[port]
            prev_status = sys_status.get('ports', {}).get(port, {}).get('status')
            if val == 'disable':
                if prev_status != 'disabled':
                    self.poe.set_port_disable_poe(port)
            elif val == 'enable':
                if prev_status == 'disabled':
                    self.poe.set_port_enable_poe(port)
            else:
                print 'Error: could not parse enable/disable value "%s". Setting swp%u to "enable".' % (val, port+1)

        for port in set_priority_ports:
            self.poe.set_port_priority(port, set_priority_ports[port])

    def parse_priority(self, swp, priority_str):
        priority_str = priority_str.lower()
        priority = PORT_CFG_PRIORITY_LOW
        if priority_str == 'high':
            priority = PORT_CFG_PRIORITY_HIGH
        elif priority_str == 'critical':
            priority = PORT_CFG_PRIORITY_CRITICAL
        elif priority_str != 'low':
            print 'Error: could not parse priority level "%s". Setting %s to "low".' % (priority_str, swp)
        return priority

class PoeResult(object):
    STATE_OK=0
    STATE_ERROR=1

    state_map = { STATE_OK:'ok', STATE_ERROR:'error' }

    def __init__(self):
        self.add_enum_maps({'state':self.state_map})
        self.state = self.STATE_OK
        self.error_str = ''

    @property
    def _error_str(self):
        return self.error_str

    @_error_str.setter
    def _error_str(self, msg):
        self.status = self.STATE_ERROR
        self.error_str = msg

    def add_enum_maps(self, new_maps):
        try:
            self._enum_maps.update(new_maps)
        except:
            self._enum_maps = new_maps

    def get_dict_str(self, dict, val):
        return dict.get(val, 'other [0x%02X' % (val))

    def __str__(self):
        outstr = ''
        for item in self.__dict__:
            if item[0] == '_':
                continue
            val_map = self._enum_maps.get(item)
            if isinstance(val_map, dict):
                outstr += '%s:%s\n' % (item, val_map.get(self.__dict__[item], 'unknown [%u]' % (self.__dict__[item])))
            else:
                outstr += '%s:%s\n' % (item, str(self.__dict__[item]))
        return outstr

    def _json(self):
        new_json = {}
        for item in self.__dict__:
            if item[0] == '_':
                continue
            val_map = self._enum_maps.get(item)
            val = self.__dict__[item]
            if val != None:
                if isinstance(val_map, dict):
                    new_json[item] = val_map.get(val, 'unknown [%s]' % (str(val)))
                else:
                    if isinstance(val, dict):
                        new_json[item] = val
                    else:
                        new_json[item] = str(val)
        return new_json

    def _mk_word_data(self, byte1, byte2):
        return byte1 * 256 + byte2

class PoePortResult(PoeResult):

    def __init__(self, port=-1):
        PoeResult.__init__(self)
        self.poe_port = port
        self.swp = 'swp%u' % (port+1)

class PoeSysStatusResult(PoeResult):

    def __init__(self):
        PoeResult.__init__(self)
        self.sys_consumed_power = 0
        self.sys_power_limit = 0
        self.ports = {}
        self.sensors = {}
        self._port_event = False
        self.cpu_status1 = 0


class PoePort4portStateResult(PoePortResult):
    RESULT_2_PAIR = '2-pair'
    RESULT_4_PAIR = '4-pair'


class PortStatusResult(PoePortResult):
    PORT_STATUS_UNKNOWN=-1
    PORT_STATUS_DISABLED=0
    PORT_STATUS_SEARCHING=1
    PORT_STATUS_DELIVERING_POWER=2
    PORT_STATUS_DENIED=3
    PORT_STATUS_FAULT=4

    PORT_STATUS_PD_TYPE_NONE=0
    PORT_STATUS_PD_TYPE_IEEE8023AF=1
    PORT_STATUS_PD_TYPE_IEEE8023AT=2
    PORT_STATUS_PD_TYPE_LEGACY=3
    PORT_STATUS_PD_TYPE_HIGH_POWER=4
    PORT_STATUS_PD_TYPE_INVALID=5
    PORT_STATUS_PD_TYPE_2PAIR_LLDP=6
    PORT_STATUS_PD_TYPE_4PAIR_LLDP=7
    PORT_STATUS_PD_TYPE_IEEE8023AF_AT=8
    PORT_STATUS_PD_TYPE_2P_NON_IEEE=9
    PORT_STATUS_PD_TYPE_4P_NON_IEEE=10
    PORT_STATUS_PD_TYPE_2P_IEEE=11
    PORT_STATUS_PD_TYPE_2P_SSPD=12
    PORT_STATUS_PD_TYPE_4P_SSPD=13
    PORT_STATUS_PD_TYPE_2P_DSPD_PHASE1=14
    PORT_STATUS_PD_TYPE_2P_DSPD=15
    PORT_STATUS_PD_TYPE_4P_DSPD=16

    port_status_map = {PORT_STATUS_UNKNOWN:'unknown',
                       PORT_STATUS_DISABLED:'disabled',
                       PORT_STATUS_SEARCHING:'searching',
                       PORT_STATUS_DELIVERING_POWER:'connected',
                       PORT_STATUS_DENIED:'power-denied',
                       PORT_STATUS_FAULT:'fault'}

    port_pd_type_map = {PORT_STATUS_PD_TYPE_NONE:'none',
                        PORT_STATUS_PD_TYPE_IEEE8023AF:'IEEE802.3af',
                        PORT_STATUS_PD_TYPE_IEEE8023AT:'IEEE802.3at',
                        PORT_STATUS_PD_TYPE_LEGACY:'legacy',
                        PORT_STATUS_PD_TYPE_HIGH_POWER:'high-power',
                        PORT_STATUS_PD_TYPE_INVALID:'invalid',
                        PORT_STATUS_PD_TYPE_2PAIR_LLDP:'IEEE802.3at',
                        PORT_STATUS_PD_TYPE_4PAIR_LLDP:'high-power',
                        PORT_STATUS_PD_TYPE_IEEE8023AF_AT:'IEEE802.3af/at',
                        PORT_STATUS_PD_TYPE_2P_NON_IEEE:'2-pair',
                        PORT_STATUS_PD_TYPE_4P_NON_IEEE:'4-pair',
                        PORT_STATUS_PD_TYPE_2P_IEEE:'2-pair',
                        PORT_STATUS_PD_TYPE_2P_SSPD:'2-pair SSPD',
                        PORT_STATUS_PD_TYPE_4P_SSPD:'4-pair SSPD',
                        PORT_STATUS_PD_TYPE_2P_DSPD_PHASE1:'2-pair DSPD P1',
                        PORT_STATUS_PD_TYPE_2P_DSPD:'2-pair DSPD',
                        PORT_STATUS_PD_TYPE_4P_DSPD:'4-pair DSPD',}

    def __init__(self, port):
        PoePortResult.__init__(self, port)
        self.add_enum_maps({'pd_type':self.port_pd_type_map,
                             'status':self.port_status_map})

class PortFullStatusResult(PortStatusResult):
    cfg_priority_map = {PORT_CFG_PRIORITY_LOW:'low',
                        PORT_CFG_PRIORITY_HIGH:'high',
                        PORT_CFG_PRIORITY_CRITICAL:'critical'}

    def __init__(self, port_num):
        PortStatusResult.__init__(self, port_num)
        self.priority = None
        self.pd_class = 'none'
        self.pd_type = self.PORT_STATUS_PD_TYPE_NONE
        self.add_enum_maps({'priority':self.cfg_priority_map })

class PortPriorityResult(PortStatusResult):

    cfg_priority_map = {PORT_CFG_PRIORITY_LOW:'low',
                        PORT_CFG_PRIORITY_HIGH:'high',
                        PORT_CFG_PRIORITY_CRITICAL:'critical'}

    def __init__(self, port_num):
        PortStatusResult.__init__(self, port_num)
        self.priority = None
        self.add_enum_maps({'priority':self.cfg_priority_map})

class SwVersionResult(PoeResult):
    def __init__(self):
        PoeResult.__init__(self)
        self.major_version = 0
        self.minor_version = 0
        self.ver_str = ''

class DiagInfoResult(PoeResult):
    def __init__(self):
        PoeResult.__init__(self)
        self.ports = {}


"""
ProgramKeyResult
Every message containing a command, test or program
key transmitted from the host to the POE controller
is followed by a Report message sent back. 
"""
class ProgramKeyResult(PoeResult):
    def __init__(self):
        PoeResult.__init__(self)
        self.result = {'PASS' :     False,
                       'BAD_CKSUM': False,
                       'EXEC_FAIL': False}
