#! /usr/bin/python
# Copyright 2019, 2020 Cumulus Networks Inc.  all rights reserved

try:
    import time
    import poe_base
    import sys
    import poe_pd69100
except ImportError, e:
    raise ImportError(str(e) + 'required module not found')

'''
802.3BT related port statuses. These status code reflect
power being delivered to a 802.3bt compliant PD.
'''

PD69200BT_PORT_STATUS_OFF_MAINS_HIGH = 0x06
PD69200BT_PORT_STATUS_OFF_MAINS_LOW = 0x07
PD69200BT_PORT_STATUS_OFF_HW_DISABLED = 0x08
PD69200BT_PORT_STATUS_OFF_PORT_UNDEFINED = 0x11
PD69200BT_PORT_STATUS_OFF_HW_FAULT = 0x12
PD69200BT_PORT_STATUS_OFF_USER = 0x1a
PD69200BT_PORT_STATUS_OFF_DETECTING = 0x1b
PD69200BT_PORT_STATUS_OFF_INVALID_DEVICE = 0x1c
PD69200BT_PORT_STATUS_OFF_UNDERLOAD = 0x1e
PD69200BT_PORT_STATUS_OFF_OVERLOAD = 0x1f
PD69200BT_PORT_STATUS_OFF_BUDGET_EXCEEDED = 0x20
PD69200BT_PORT_STATUS_OFF_BAD_CAP = 0x25
PD69200BT_PORT_STATUS_OFF_PORT_HOT = 0x35
PD69200BT_PORT_STATUS_OFF_DEVICE_HOT = 0x36
PD69200BT_PORT_STATUS_OFF_PM_STATIC = 0x3c
PD69200BT_PORT_STATUS_OFF_CLASS_ERR = 0x43
PD69200BT_PORT_STATUS_2P_NON_IEEE = 0x80
PD69200BT_PORT_STATUS_2P_IEEE = 0x81
PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE_LEGACY = 0x82
PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE = 0x83
PD69200BT_PORT_STATUS_4P_ON_4P_NON_IEEE = 0x84
PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_SSPD = 0x85
PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_SSPD = 0x86
PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD_PHASE1 = 0x87
PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD = 0x88
PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_DSPD = 0x89
PD69200BT_PORT_STATUS_OPEN = 0xa8
# SUB1 field unique to 802.3BT firmware
PD69200BT_PKT_SUB1_BT_PORT_STATUS = 0xc1
PD69200BT_PKT_SUB1_BT_PORT_CLASS = 0xc4
PD69200BT_PKT_SUB1_ALTB_POWER_UP = 0x52
PD69200BT_PKT_SUB1_GET_BT_LAYER2_PD=0x54
PD69200BT_PKT_SUB1_BT_MEASUREMENTS = 0xc5
PD69200BT_PKT_SUB1_SYSTEM_STATUS = 0xd0
PD69200BT_PKT_SUB1_PORT_CFG1 = 0xc0
PD69200BT_PKT_SUB1_BT_EVENT = 0xd1
PD69200BT_PKT_SUB1_BT_DEV_INFO = 0xd3
# Packet data fields
PD69200BT_PKT_DATA_SETTINGS_UNCHANGED = 0xff
# PM modes specific to BT
PD69200BT_PM_MODE_DYNAMIC = 0x00
PD69200BT_PM_MODE_TPPL_BT = 0xf1
PD69200BT_PM_MODE_DYNAMIC_AND_TPPL = 0xf2
'''
POE chip specific base class for 802.3bt standard.
provides access to the following POE chips:
               MicroSemi PD69200
Starting from FW revision 3.xx, PD69200 supports
802.3bt standard. The new firmware revision has some
commands that have changed from the af/at version (ver 2.xx).
We overload methods that are different in the 3.xx firmware.
'''
pd69200bt_port_error_map = {
    PD69200BT_PORT_STATUS_OFF_MAINS_HIGH : 'mains high',
    PD69200BT_PORT_STATUS_OFF_MAINS_LOW  : 'mains low',
    PD69200BT_PORT_STATUS_OFF_HW_DISABLED : 'hw disabled',
    PD69200BT_PORT_STATUS_OFF_PORT_UNDEFINED : 'port undefined',
    PD69200BT_PORT_STATUS_OFF_HW_FAULT : 'hw fault',
    PD69200BT_PORT_STATUS_OFF_INVALID_DEVICE : 'invalid pd',
    PD69200BT_PORT_STATUS_OFF_UNDERLOAD : 'underload',
    PD69200BT_PORT_STATUS_OFF_OVERLOAD : 'overload',
    PD69200BT_PORT_STATUS_OFF_BUDGET_EXCEEDED : 'power budget exceeded',
    PD69200BT_PORT_STATUS_OFF_BAD_CAP : 'detection failed',
    PD69200BT_PORT_STATUS_OFF_PORT_HOT : 'port over temp',
    PD69200BT_PORT_STATUS_OFF_DEVICE_HOT : 'device hot',
    PD69200BT_PORT_STATUS_OFF_PM_STATIC : 'power limit exceeded',
    PD69200BT_PORT_STATUS_OFF_CLASS_ERR : 'invalid class',
}

pd69200bt_port_ok_results = {
    PD69200BT_PORT_STATUS_OFF_DETECTING : poe_base.PortStatusResult.PORT_STATUS_SEARCHING,
    PD69200BT_PORT_STATUS_OPEN : poe_base.PortStatusResult.PORT_STATUS_SEARCHING,
    PD69200BT_PORT_STATUS_OFF_USER : poe_base.PortStatusResult.PORT_STATUS_DISABLED,
    PD69200BT_PORT_STATUS_2P_NON_IEEE : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_IEEE : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_4P_ON_4P_NON_IEEE : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_SSPD : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_SSPD : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD_PHASE1 : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_DSPD : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
    PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE_LEGACY : poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER,
}
 
pd69200bt_port_pd_type_map = {
    PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE_LEGACY:poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_LEGACY,
    PD69200BT_PORT_STATUS_2P_NON_IEEE : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_NON_IEEE,
    PD69200BT_PORT_STATUS_2P_IEEE : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_IEEE,
    PD69200BT_PORT_STATUS_2P_ON_4P_NON_IEEE : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_NON_IEEE,
    PD69200BT_PORT_STATUS_4P_ON_4P_NON_IEEE : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_4P_NON_IEEE,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_SSPD : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_SSPD,
    PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_SSPD : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_4P_SSPD,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD_PHASE1 : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_DSPD_PHASE1,
    PD69200BT_PORT_STATUS_2P_ON_4P_IEEE_DSPD : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_2P_DSPD,
    PD69200BT_PORT_STATUS_4P_ON_4P_IEEE_DSPD : poe_base.PortFullStatusResult.PORT_STATUS_PD_TYPE_4P_DSPD,
}

class PoePD69200BT(poe_pd69100.PoePD69100):
    def __init__(self):
        poe_pd69100.PoePD69100.__init__(self)

    def _get_single_port_status(self, port):
        result = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                            poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                            PD69200BT_PKT_SUB1_BT_PORT_STATUS, port],
                                           poe_pd69100.PD69100PortFullStatusResult(port),
                                           poe_pd69100.PD69100PortFullStatusResult.PORT_STATUS_RESPONSE)
        return result.status

    def reset_ports(self, portList):
        self.log_debug('PoePD69200BT::reset_ports(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        reset_ports = []
        for port in portList:
            status = self._get_single_port_status(port)
            if status == poe_base.PortStatusResult.PORT_STATUS_DISABLED:
                continue
            reset_ports.append(port)
        self.set_port_disable_poe(reset_ports)
        time.sleep(5)
        self.set_port_enable_poe(reset_ports)
        
    def get_poe_diag_info(self):
        self.log_debug('PoePD69200BT::get_poe_diag_info')
        result = poe_pd69100.PD69100GetDiagInfoResult()

        ''' get power info
        '''
        request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                         poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                         poe_pd69100.PD69100_PKT_SUB1_SUPPLY,
                                         poe_pd69100.PD69100_PKT_SUB2_MAIN])
        response = self._send_request(request)
        result.handle_response(response, poe_pd69100.PD69100GetDiagInfoResult.SUPPLY_RESPONSE)

        ''' get power management info
        '''
        request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                         poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                         poe_pd69100.PD69100_PKT_SUB1_SUPPLY,
                                         poe_pd69100.PD69100_PKT_SUB2_POWER_MGMT_MODE])
        response = self._send_request(request)
        result.handle_response(response, poe_pd69100.PD69100GetDiagInfoResult.MODE_RESPONSE)

        ''' get individual mask info
        '''
        masks = (0x00, 0x01, 0x08, 0x0c, 0x11, 0x14, 0x15, 0x1b, 0x1e, 0x1f, 0x28, 0x29, 0x2a, 0x2b, 0x2d, 0x2e, 0x2f,
                 0x30, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x43 )
        for mask in masks:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                             poe_pd69100.PD69100_PKT_SUB1_INDIVIDUAL_MASK,
                                             mask])
            response = self._send_request(request)
            result.handle_response(response, poe_pd69100.PD69100GetDiagInfoResult.MASK_RESPONSE, mask)

        ''' get power-up timeout setting for all devices
        '''
        for device in range(0x0c):
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_TEST,
                                             poe_pd69100.PD69100_PKT_SUB1_READ_RONS,
                                             0, 0x99, device])
            response = self._send_request(request)
            result.handle_response(response, poe_pd69100.PD69100GetDiagInfoResult.REG_RESPONSE, device)

        ''' get port info
        '''
        for port in range(48):
            port_result = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                                     poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                                     PD69200BT_PKT_SUB1_BT_PORT_STATUS, port],
                                                    PD69200BTGetPortDiagInfoResult(port),
                                                    PD69200BTGetPortDiagInfoResult.STATUS_RESPONSE)
            self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST,
                                       0x00,
                                       poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                       PD69200BT_PKT_SUB1_BT_PORT_CLASS, port],
                                      port_result,
                                      PD69200BTGetPortDiagInfoResult.CLASS_RESPONSE)

            self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00, poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                       poe_pd69100.PD69100_PKT_SUB1_SUPPLY, port],
                                      port_result, PD69200BTGetPortDiagInfoResult.POWER_LIMIT_RESPONSE)
            self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00, poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                       PD69200BT_PKT_SUB1_GET_BT_LAYER2_PD, port],
                                      port_result, PD69200BTGetPortDiagInfoResult.L2_POWER_RESPONSE)
            result.ports[port] = port_result._json()
        return result._json()


    def get_poe_system_status(self, incremental=False):
        self.log_debug('PoePD69200BT::get_poe_system_status(%s)' % (str(incremental)))
        result = PD69200BTGetSysStatusResult()
        request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                        poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                        PD69200BT_PKT_SUB1_SYSTEM_STATUS])

        response = self._send_request(request)
        # Check if event happened.
        if response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA10] & 1:
            # BT event command to get further information on events.
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                             PD69200BT_PKT_SUB1_BT_EVENT])
            events_response = self._send_request(request)
            result.handle_response(events_response, PD69200BTGetSysStatusResult.EVENT_RESPONSE)

        port_results = {}
        for port in range(48):
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                            poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                            PD69200BT_PKT_SUB1_BT_PORT_STATUS, port])
            response = self._send_request(request)
            port_result = PD69200BTPortStatusResult(port)
            port_result.handle_response(response)
            port_results[port] = port_result._json()
        result.ports = port_results

        request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                         poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                         poe_pd69100.PD69100_PKT_SUB1_SUPPLY, poe_pd69100.PD69100_PKT_SUB2_MAIN])
        response = self._send_request(request)
        result.handle_response(response, PD69200BTGetSysStatusResult.SUPPLY_RESPONSE)

        for device_id in range(12):
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                             PD69200BT_PKT_SUB1_BT_DEV_INFO, device_id])
            response = self._send_request(request)
            result.handle_response(response, PD69200BTGetSysStatusResult.TEMP_RESPONSE)

        return result._json()

    def get_port_extra_status(self, portList):
        self.log_debug('PoePD69200BT::get_port_extra_status(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        results = {}
        for port in portList:
            result = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                                poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                                PD69200BT_PKT_SUB1_BT_PORT_STATUS, port],
                                               PD69200BTPortFullStatusResult(port),
                                               PD69200BTPortFullStatusResult.PORT_STATUS_RESPONSE)
            self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                       poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                       PD69200BT_PKT_SUB1_BT_MEASUREMENTS, port],
                                      result, PD69200BTPortFullStatusResult.MEASUREMENTS_RESPONSE)
            self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                       poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                       PD69200BT_PKT_SUB1_PORT_CFG1, port],
                                      result, PD69200BTPortFullStatusResult.PRIORITY_RESPONSE)
            # the PD69200 sometimes reports non-zero values for current, power, voltage on non-connected ports
            if result.status != poe_base.PortStatusResult.PORT_STATUS_DELIVERING_POWER:
                result.current = 0
                result.power = 0
                result.voltage = 0

            results[port] = result._json()
        return results

    def get_poe_system_firmware_status(self):
        self.log_debug('PoePD69200BT::get_poe_system_firmware_status')
        result = PD69100GetSysStatusResult()
        request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_REQUEST,
                                         0x00,
                                         poe_pd69100.PD69100_PKT_SUB_GLOBAL,
                                         PD69200BT_PKT_SUB1_SYSTEM_STATUS])
        response = self._send_request(request)
        result.handle_response(response, poe_pd69100.PD69100GetSysStatusResult.CPU_STATUS1_RESPONSE)
        return result.cpu_status1

    def get_port_priority(self, portList):
        self.log_debug('PoePD69200BT::get_port_priority(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        response_proc_obj = PD69200BTPortFullStatusResult()
        results = {}
        for port in portList:
            results[port] = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
                                                       poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                                       PD69200BT_PKT_SUB1_PORT_CFG1, port],
                                                      response_proc_obj._json, PD69200BTPortFullStatusResult.PRIORITY_RESPONSE)
        return results

    def set_port_priority(self, port_list, level):
        self.log_debug('PoePD69200BT::set_port_priority(%s)' % (str(port_list)))
        port_list = self._mk_list(port_list)
        priority = poe_pd69100.PD69100_PKT_DATA_PRIORITY_LOW
        for our_val, base_val in poe_pd69100.pd69100_priority_map.items():
            if base_val == level:
                priority = our_val
        for port in port_list:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_PORT_CFG1, port,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             priority])
            self._send_request(request)

    def set_port_disable_poe(self, port_list):
        self.log_debug('PoePD69200BT::set_port_disable_poe(%s)' % (str(port_list)))
        port_list = self._mk_list(port_list)
        for port in port_list:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_PORT_CFG1, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDDISABLE,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED])
            self._send_request(request)
            
    def set_port_enable_poe(self, portList):
        self.log_debug('PoePD69200BT::set_port_enable_poe(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_PORT_CFG1, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDENABLE,
                                             PD69200BT_PM_MODE_DYNAMIC,
                                             0x10,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED])
            self._send_request(request)

    def set_pm_settings(self, portList):
        self.log_debug('PoePD69200BT::set_pm_settings(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_PORT_CFG1, port,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PM_MODE_DYNAMIC,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED])
            self._send_request(request)
            
    def set_enable_4pair_mode(self, portList):
        '''
        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_PORT_CFG1,
                                             port,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             0x10,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED,
                                             PD69200BT_PKT_DATA_SETTINGS_UNCHANGED])
            self._send_request(request)
        '''
        pass

    def get_port_4pair_state(self, portList):
        self.log_debug('PoePD69200BT::get_ports_4pair_state()')
        portList = self._mk_list(portList)

        results = {}
        for port in portList:
            res = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST,
                                             0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_ALTB_POWER_UP,
                                             port], PD69200BTGetPort4pairStateResult(port))
            results[port] = res._json()
        return results

    def get_ports_4pair_config(self):
        self.log_debug('PoePD69100::get_port_4pair_config()')
        result = {}
        res = self._send_request(self._new_request_str([PD69100_PKT_KEY_REQUEST,
                                                   0x00,
                                                   PD69100_PKT_SUB_GLOBAL,
                                                   PD69100_PKT_SUB1_EN_DIS_4PAIR]))
	for port in range(48):
            result[port] = '4pair' 
        return result


    def set_enable_4pair_power(self, portList):
        '''
        self.log_debug('PoePD69200BT::set_enable_4pair_power(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_ALTB_POWER_UP,
                                             port,
                                             poe_pd69100.PD69100_ALTB_PUP_CMD_ENABLE])
            self._send_request(request)
        '''
        pass

    def set_disable_4pair_power(self, portList):
        '''
        self.log_debug('PoePD69200BT::set_disable_4pair(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            status = self._get_single_port_status(port)
            if status != PD69200BTPortFullStatusResult.PORT_STATUS_DISABLED:
                self.set_port_disable_poe(port)
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             PD69200BT_PKT_SUB1_ALTB_POWER_UP,
                                             port,
                                             poe_pd69100.PD69100_ALTB_PUP_CMD_DISABLE])
            self._send_request(request)
            if status != PD69200BTPortFullStatusResult.PORT_STATUS_DISABLED:
                self.set_port_enable_poe(portList)
        '''
        pass

    def set_port_l2_power(self, portList, requested, allocated):
        pass


class PD69200BTPortStatusResult(poe_base.PortStatusResult):
    def handle_response(self, response):
        port_byte = response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]
        self.status = pd69200bt_port_ok_results.get(port_byte, poe_base.PortStatusResult.PORT_STATUS_FAULT)
        if self.status == poe_base.PortStatusResult.PORT_STATUS_FAULT:
            self._error_str = pd69200bt_port_error_map.get(port_byte, 'other [0x%02X]' % (port_byte))
        elif self.status == poe_base.PortFullStatusResult.PORT_STATUS_DELIVERING_POWER:
            self.pd_type = pd69200bt_port_pd_type_map.get(port_byte, port_byte)

class PD69200BTPortFullStatusResult(poe_base.PortFullStatusResult):
    MEASUREMENTS_RESPONSE=1
    PORT_STATUS_RESPONSE=2
    PRIORITY_RESPONSE=3

    def handle_response(self, response, response_type):
        if response_type == self.PORT_STATUS_RESPONSE:
            en_dis = response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA1]
            if en_dis == poe_pd69100.PD69100_PKT_DATA_PORT_DISABLED:
                self.status = poe_base.PortFullStatusResult.PORT_STATUS_DISABLED
            else:
                port_byte = response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]
                self.status = pd69200bt_port_ok_results.get(port_byte, poe_base.PortFullStatusResult.PORT_STATUS_FAULT)
                if self.status == poe_base.PortFullStatusResult.PORT_STATUS_FAULT:
                    self._error_str = pd69200bt_port_error_map.get(port_byte, 'other [0x%02X]' % (port_byte))
                elif self.status == poe_base.PortFullStatusResult.PORT_STATUS_DELIVERING_POWER:
                    class_hex = hex(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA2])
                    if class_hex[-1] is 'c':
                        self.pd_class = class_hex[-2]
                    else:
                        self.pd_class = class_hex[2:]
                    self.pd_type = pd69200bt_port_pd_type_map.get(port_byte, port_byte)
        elif response_type == self.MEASUREMENTS_RESPONSE:
            self.current = self._mk_word_data(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA2],
                                              response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA3])
            self.power = self._mk_word_data(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA4],
                                            response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA5]) * 0.1
            val = self._mk_word_data(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA7],
                                     response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA8])
            if val == 65535:
                val = 0
            self.voltage = val/10.0      # Volts

        elif response_type == self.PRIORITY_RESPONSE:
            self.priority = poe_pd69100.pd69100_priority_map.get(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA5])

            
class PD69200BTGetPort4pairStateResult(poe_base.PoePort4portStateResult):
    def handle_response(self, response, type=None):
        self.pair_mode = self.RESULT_4_PAIR


class PD69200BTGetPortDiagInfoResult(poe_base.PoePortResult):
    STATUS_RESPONSE=1
    POWER_LIMIT_RESPONSE=2
    L2_POWER_RESPONSE=3
    CLASS_RESPONSE = 4

    def handle_response(self, response, response_type):
        if response_type == self.STATUS_RESPONSE:
            self.status_enabled = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA1]
            self.status_status = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]
            self.status_force_power = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]
            self.status_latch = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA3]
            self.status_af_at = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA8]
        elif response_type == self.POWER_LIMIT_RESPONSE:
            self.power_ppl = 'not supported'
        elif response_type == self.L2_POWER_RESPONSE:
            self.l2_allocated = '%.1fw' % ((response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0] * 256 +
                                            response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA1])/10)
            self.l2_requested = '%.1fw' % ((response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA2] * 256 +
                                            response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA3])/10)
            self.l2_actual = '%.1fw' % (((response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA9] * 256 +
                                          response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA10]) & 0xfff)/10)
        elif response_type == self.CLASS_RESPONSE:
            self.status_4pair = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA1]
            self.status_class = '0x%02X' % response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA6]
            self.power_tppl = '0x%04X' % (response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA7] * 256 +
                                          response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA8])


class PD69200BTGetSysStatusResult(poe_base.PoeSysStatusResult):
    INTERRUPTS_RESPONSE=1
    SUPPLY_RESPONSE=2
    POWER_RESPONSE=3
    TEMP_RESPONSE=4
    CPU_STATUS1_RESPONSE=5
    EVENT_RESPONSE = 6

    def handle_response(self, response, response_type):
        if response_type == self.EVENT_RESPONSE:
            for i in xrange(6):
                if response[i] > 0:
                    self._port_event = True
        elif response_type == self.SUPPLY_RESPONSE:
            self.sys_consumed_power = self._mk_word_data(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0],
                                                         response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA1])
            self.sys_power_limit = self._mk_word_data(response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA8],
                                                      response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA9])
        elif response_type == self.TEMP_RESPONSE:
            temp = response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA7]
            if temp != 255:
                self.sensors[response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]] = temp * 1000
        elif response_type == self.CPU_STATUS1_RESPONSE:
            self.cpu_status1 = response[poe_pd69100.PD69100_PKT_OFFSET_RXDATA0]
