#! /usr/bin/python
# Copyright 2015,2016,2019 Cumulus Networks Inc.  all rights reserved

#############################################################################
# Platform and model specific poe subclass, inherits from the base poe class,
# and provides the followings:
# - loads the chip-specific poe implementation
#

try:
    import exceptions
    import os
    import termios
    import time
    import select
    import errno
    import fcntl
    import struct
    import serial
    import poe_base
    import poe_bcm591xx
    import syslog
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def poe():
    return poe_as4610_54p()

class poe_as4610_54p(poe_bcm591xx.PoeBcm591xx):
    cpld_path = '/sys/bus/i2c/devices/0-0030'

    def __init__(self):
        poe_bcm591xx.PoeBcm591xx.__init__(self, poe_bcm591xx.BCM591xx_CHIP_59121)
        self.log_debug('poe_as4610_54p::init()')
        self._fd = None

    def poe_enable(self):
        self._enable_poe_in_cpld()

    def poe_disable(self):
        self._disable_poe_in_cpld()

    def handle_warm_cold_boot(self):
        completion_flag = False
        failed_count = 0
        while not completion_flag:
            try:
                self._do_default_setup()
            except Exception as default_setup_err:
                failed_count += 1
                syslog.syslog(syslog.LOG_ERR, '_do_default_setup failed')
                if failed_count > 3:
                    syslog.syslog(syslog.LOG_ERR, '_do_default_setup failed more than 3 times, poed exiting')
                    raise default_setup_err
                time.sleep(2)
            else:
                completion_flag = True
        poe_bcm591xx.PoeBcm591xx.handle_warm_cold_boot(self)

    def handle_poed_start(self):
        poe_bcm591xx.PoeBcm591xx.handle_poed_start(self)
        self.poe_enable()

    @property
    def fd(self):
        if not self._fd:
            self._fd = serial.Serial('/dev/ttyS1', baudrate=19200, bytesize=8, parity='N', stopbits=1, timeout=3)
        return self._fd

    def get_poe_params(self):
        self.log_debug('poe_as4610_54p::get_poe_params()')
        try:
            if open(os.path.join(self.cpld_path, 'poe_support')).read().strip() == '1':
                # POE+ and UPOE are supported on this platform. UPOE is only for the last 8 ports (41-48)
                ports = {}
                for port_id in range(40):
                    ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                            True, poe_base.MAX_POEPLUS_POWER, False, 0)
                for port_id in range(40, 48):
                    ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                            True, poe_base.MAX_UPOE_POWER, False, 0)
                return poe_base.PoeParams(True, 5, ports)
        except:
            pass
        # POE is not supported
        return poe_base.PoeParams()

    def adjust_system_poe_power(self, prev_index=0):
        self.log_debug('poe_as4610_54p::adjust_system_poe_power()')
        pass

    def poe_write(self, data):
        self.log_debug('poe_as4610_54p::write()')
        self.fd.write(struct.pack('B' * len(data), *data))

    def poe_read(self, size=1):
        self.log_debug('poe_as4610_54p::read()')
        read_data = self.fd.read(size)
        return struct.unpack('B' * len(read_data), read_data)

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)

    def _enable_poe_in_cpld(self):
        open(os.path.join(self.cpld_path, 'enable_poe'), 'w').write('1')

    def _disable_poe_in_cpld(self):
        open(os.path.join(self.cpld_path, 'enable_poe'), 'w').write('0')

    def _do_default_setup(self):
        port_map = [
            (0x00, 0x07, 0x06, 0x06, 0x00, 0x00, 0x00), # port, 4pair, dev_id, primary, alt, powerup, detect, mode
            (0x01, 0x07, 0x07, 0x07, 0x00, 0x00, 0x00),
            (0x02, 0x07, 0x04, 0x04, 0x00, 0x00, 0x00),
            (0x03, 0x07, 0x05, 0x05, 0x00, 0x00, 0x00),
            (0x04, 0x07, 0x02, 0x02, 0x00, 0x00, 0x00),
            (0x05, 0x07, 0x03, 0x03, 0x00, 0x00, 0x00),
            (0x06, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00),
            (0x07, 0x07, 0x01, 0x01, 0x00, 0x00, 0x00),
            (0x08, 0x06, 0x06, 0x06, 0x00, 0x00, 0x00),
            (0x09, 0x06, 0x07, 0x07, 0x00, 0x00, 0x00),
            (0x0A, 0x06, 0x04, 0x04, 0x00, 0x00, 0x00),
            (0x0B, 0x06, 0x05, 0x05, 0x00, 0x00, 0x00),
            (0x0C, 0x06, 0x02, 0x02, 0x00, 0x00, 0x00),
            (0x0D, 0x06, 0x03, 0x03, 0x00, 0x00, 0x00),
            (0x0E, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00),
            (0x0F, 0x06, 0x01, 0x01, 0x00, 0x00, 0x00),
            (0x10, 0x05, 0x06, 0x06, 0x00, 0x00, 0x00),
            (0x11, 0x05, 0x07, 0x07, 0x00, 0x00, 0x00),
            (0x12, 0x05, 0x04, 0x04, 0x00, 0x00, 0x00),
            (0x13, 0x05, 0x05, 0x05, 0x00, 0x00, 0x00),
            (0x14, 0x05, 0x02, 0x02, 0x00, 0x00, 0x00),
            (0x15, 0x05, 0x03, 0x03, 0x00, 0x00, 0x00),
            (0x16, 0x05, 0x00, 0x00, 0x00, 0x00, 0x00),
            (0x17, 0x05, 0x01, 0x01, 0x00, 0x00, 0x00),
            (0x18, 0x04, 0x06, 0x06, 0x00, 0x00, 0x00),
            (0x19, 0x04, 0x07, 0x07, 0x00, 0x00, 0x00),
            (0x1A, 0x04, 0x04, 0x04, 0x00, 0x00, 0x00),
            (0x1B, 0x04, 0x05, 0x05, 0x00, 0x00, 0x00),
            (0x1C, 0x04, 0x02, 0x02, 0x00, 0x00, 0x00),
            (0x1D, 0x04, 0x03, 0x03, 0x00, 0x00, 0x00),
            (0x1E, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00),
            (0x1F, 0x04, 0x01, 0x01, 0x00, 0x00, 0x00),
            (0x20, 0x03, 0x06, 0x06, 0x00, 0x00, 0x00),
            (0x21, 0x03, 0x07, 0x07, 0x00, 0x00, 0x00),
            (0x22, 0x03, 0x04, 0x04, 0x00, 0x00, 0x00),
            (0x23, 0x03, 0x05, 0x05, 0x00, 0x00, 0x00),
            (0x24, 0x03, 0x02, 0x02, 0x00, 0x00, 0x00),
            (0x25, 0x03, 0x03, 0x03, 0x00, 0x00, 0x00),
            (0x26, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00),
            (0x27, 0x03, 0x01, 0x01, 0x00, 0x00, 0x00),
            (0x28, 0x02, 0x04, 0x05, 0x01, 0x02, 0x01),
            (0x29, 0x02, 0x06, 0x07, 0x01, 0x02, 0x01),
            (0x2A, 0x02, 0x00, 0x01, 0x01, 0x02, 0x01),
            (0x2B, 0x02, 0x02, 0x03, 0x01, 0x02, 0x01),
            (0x2C, 0x01, 0x04, 0x05, 0x01, 0x02, 0x01),
            (0x2D, 0x01, 0x06, 0x07, 0x01, 0x02, 0x01),
            (0x2E, 0x01, 0x00, 0x01, 0x01, 0x02, 0x01),
            (0x2F, 0x01, 0x02, 0x03, 0x01, 0x02, 0x01),
        ]
        try:
            self._handleSysCommand(poe_bcm591xx.BCM591xx_CMD_SET_LOGICAL_MAP_ENABLE, [poe_bcm591xx.BCM591xx_POE_ENABLE])
            for cmds in port_map:
                self._handleSysCommand(0x0E, cmds)
            self._handleSysCommand(poe_bcm591xx.BCM591xx_CMD_SET_POWER_MODE, [poe_bcm591xx.BCM591xx_MODE_STATIC_WITH_PRIORITY])

            all_ports = list(range(48))
            self.set_port_priority(all_ports, poe_base.PORT_CFG_PRIORITY_LOW)
            self._handleQuadPortCommand(all_ports, poe_bcm591xx.BCM591xx_CMD_SET_PORT_DETECTION_TYPE, poe_bcm591xx.BCM591xx_DETECT_TYPE_4P_PLUS_LEGACY)
            self._handleQuadPortCommand(all_ports, poe_bcm591xx.BCM591xx_CMD_SET_PORT_PWR_THRESHOLD_TYPE, poe_bcm591xx.BCM591xx_PWR_THRESHOLD_TYPE_CLASS_BASED)
            self._handleQuadPortCommand(all_ports, poe_bcm591xx.BCM591xx_CMD_SET_PORT_RESET, poe_bcm591xx.BCM591xx_PORT_RESET)
            time.sleep(0.2)
            guard_band = 200
            level = 7500
            self._handleSysCommand(poe_bcm591xx.BCM591xx_CMD_SET_PWR_SOURCE_CFG, [0, level >> 8, level & 0xff, guard_band >> 8, guard_band & 0xff])
            self._handleSysCommand(poe_bcm591xx.BCM591xx_CMD_SET_PWR_SOURCE_CFG, [1, level >> 8, level & 0xff, guard_band >> 8, guard_band & 0xff])
            level = 15000
            self._handleSysCommand(poe_bcm591xx.BCM591xx_CMD_SET_PWR_SOURCE_CFG, [2, level >> 8, level & 0xff, guard_band >> 8, guard_band & 0xff])
        except:
            raise
