#! /usr/bin/python
# Copyright 2019 Cumulus Networks, Inc.

#############################################################################
# Platform and model specific poe subclass, inherits from the base poe class,
# and provides the followings:
# - loads the chip-specific poe implementation
#

try:
    import exceptions
    import os
    import termios
    import time
    import select
    import errno
    import fcntl
    import struct
    import serial
    import poe_base
    import poe_pd69100
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def poe():
    return poe_dell3048()

class poe_dell3048(poe_pd69100.PoePD69100):
    ''' according to dni3048 spec. POE port settings are:
          port             - /dev/ttyS1
          baud rate        - 19200 bps
          stop bits        - 1
          parity           - none
          data bits        - 8
          flow control     - none
          interpacket gap  - > 20ms
          timeout          - 50ms
          byte gap         - < 2ms
    '''
    def __init__(self):
        poe_pd69100.PoePD69100.__init__(self)
        self.log_debug('poe_dni3048t::init()')
        self._fd = None
        self._portModes = {}

    def poe_enable(self):
        self._enable_poe_in_cpld()
        
    def set_port_enable_poe(self, portList):
        self.log_debug('PoePD69100::set_port_enable_poe(%s)' % (str(portList)))

        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             poe_pd69100.PD69100_PKT_SUB1_PORTFULLINIT, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDENABLE,
                                             0x4e])
            self._send_request(request)

    def set_port_disable_poe(self, portList):
        self.log_debug('PoePD69100::set_port_disable_poe(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             poe_pd69100.PD69100_PKT_SUB1_PORTFULLINIT, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDDISABLE,
                                             0x4e])
            self._send_request(request)

    def reset_ports(self, portList):
        self.log_debug('PoePD69100::reset_ports(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        reset_ports = []
        for port in portList:
            result = self._handle_port_command([poe_pd69100.PD69100_PKT_KEY_REQUEST, 0x00,
						poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                                poe_pd69100.PD69100_PKT_SUB1_PORTSTATUS, port],
                                                poe_pd69100.PD69100PortFullStatusResult(port),
                                                poe_pd69100.PD69100PortFullStatusResult.PORT_STATUS_RESPONSE)
            if result.status == result.PORT_STATUS_DISABLED:
                continue

            reset_ports.append(port)
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             poe_pd69100.PD69100_PKT_SUB1_PORTFULLINIT, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDDISABLE,
                                             0x4e])
            self._send_request(request)

        time.sleep(5)
        for port in reset_ports:
            request = self._new_request_str([poe_pd69100.PD69100_PKT_KEY_CMD, 0x00,
                                             poe_pd69100.PD69100_PKT_SUB_CHANNEL,
                                             poe_pd69100.PD69100_PKT_SUB1_PORTFULLINIT, port,
                                             poe_pd69100.PD69100_PKT_DATA_CMDENABLE,
                                             0x4e])
            self._send_request(request)

    def set_enable_4pair_power(self, portList):
        self.log_debug('PoePD69100::set_enable_4pair_power(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            self._portModes[port] = poe_base.PoePort4portStateResult.RESULT_4_PAIR

    def set_disable_4pair_power(self, portList):
        self.log_debug('PoePD69100::set_disable_4pair(%s)' % (str(portList)))
        portList = self._mk_list(portList)
        for port in portList:
            self._portModes[port] = poe_base.PoePort4portStateResult.RESULT_2_PAIR

    def get_port_4pair_state(self, portList):
        '''
        checks for current hw_pair mode
        '''
        self.log_debug('PoePD69100::get_ports_4pair_state()')
        portList = self._mk_list(portList)

        results = {}
        for port in portList:
            results[port] = { 'pair_mode':self._portModes.get(port, poe_base.PoePort4portStateResult.RESULT_2_PAIR) }
        return results

    def handle_warm_cold_boot(self):
        port_map = {
             0:(0,  1),  1:(2,  3),  2:(4, 5),   3:(6,  7),
             4:(8, 9),  5:(10, 11),  6:(12, 13),  7:(14, 15),
             8:(16, 17),  9:(18, 19), 10:(20, 21), 11:(22, 23),
            12:(24, 255), 13:(25, 255), 14:(26, 255), 15:(27, 255),
            16:(28, 255), 17:(29, 255), 18:(30, 255), 19:(31, 255),
            20:(32, 255), 21:(33, 255), 22:(34, 255), 23:(35, 255),
            24:(36, 255), 25:(37, 255), 26:(38, 255), 27:(39, 255),
            28:(40, 255), 29:(41, 255), 30:(42, 255), 31:(43, 255),
            32:(44, 255), 33:(45, 255), 34:(46, 255), 35:(47, 255),
            36:(48, 255), 37:(49, 255), 38:(50, 255), 39:(51, 255),
            40:(52, 255), 41:(53, 255), 42:(54, 255), 43:(55, 255),
            44:(56, 255), 45:(57, 255), 46:(58, 255), 47:(59, 255)
        }
        for port_id, port_phys in port_map.items():
        	self.set_port_temp_matrix(port_id, port_phys[0], port_phys[1])
		
        self.set_active_matrix()


        ''' turn on L2 management
        '''
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_LAYER2_LLDP, 1)


        ''' set temp thresholds
              140C => 0x8c
        '''
        for device in range(8):
            self.set_device_temp_threshold(device, 0x8c)

        self._persist_settings()

        ''' this will apply the saved configuration
        '''
        poe_pd69100.PoePD69100.handle_warm_cold_boot(self)

    def handle_poed_start(self):
        poe_pd69100.PoePD69100.handle_poed_start(self)
        '''
            enable POE in the CPLD
        '''
        self.enable_poe_in_cpld()

    @property
    def fd(self):
        if not self._fd:
            self._fd = serial.Serial('/dev/ttyS1', baudrate=19200, bytesize=8, parity='N', stopbits=1, timeout=1)
        return self._fd

    def enable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/0-0028/poe_plus_power', 'w').write('1')

    def disable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/0-0028/poe_plus_power', 'w').write('0')

    def poe_write(self, data):
        self.log_debug('poe_dni3048t::write()')
        return self.fd.write(struct.pack('B' * len(data), *data))

    def poe_write_unpacked(self, data):
        self.log_debug('poe_dni3048t::write_unpacked()')
        return self.fd.write(data)

    def poe_read(self, size=1):
        self.log_debug('poe_dni3048t::read()')
        read_data = self.fd.read(size)
        return struct.unpack('B' * len(read_data), read_data)

    def poe_read_unpacked(self, size=1):
        self.log_debug('poe_dni3048t::read_unpacked()')
        read_data = self.fd.read(size)
        return read_data

    def get_poe_params(self):
        ports = {}
        for port_id in range(12):
            ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                    True, poe_base.MAX_POEPLUS_POWER, True, poe_base.MAX_UPOE_POWER)
        for port_id in range(12, 48):
            ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                    True, poe_base.MAX_POEPLUS_POWER, True, poe_base.MAX_POEPLUS_POWER)
        sensors = {}
        for sensor_id in range(8):
            sensors[sensor_id] = poe_base.PoeSensorParams('temp%u' % (sensor_id + 1),
                                                          sensor_id, 120000, 150000)
        return poe_base.PoeParams(True, 1, ports, sensors)

    def adjust_system_poe_power(self, prev_index=-1):
        power_bank_array = [
        #   PB12  PB13  PB14
            (0,      0,   0),    # 0: no power supplies, never used
            (550,    0, 550),    # 1: PSU1 715
            (900,    0, 900),    # 2: PSU1 1100
            (550,  550,   0),    # 3: PSU2 715
            (1200, 550, 550),    # 4: PSU1 715,  PSU2 715
            (1500, 550, 900),    # 5: PSU1 1100, PSU2 715
            (900,  900,   0),    # 6: PSU2 1100
            (1500, 900, 550),    # 7: PSU1 715, PSU2 1100
            (1800, 900, 900),    # 8: PSU1 1100, PSU2 1100
        ]

        power_bank = (12, 13, 14)
        psu1_val = 0
        psu2_val = 0
        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr1_all_ok').read()) == 1:
                psu1_val = 1
                inp = open('/sys/bus/i2c/devices/5-0052/eeprom').read()[:5]
                if inp == '1100W':
                    psu1_val = 2
        except:
            pass

        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr2_all_ok').read()) == 1:
                inp = open('/sys/bus/i2c/devices/6-0052/eeprom').read()[:5]
                psu2_val = 1
                if inp == '1100W':
                    psu2_val = 2
        except:
            pass

        index = psu2_val * 3 + psu1_val
        if index == 0:
            return prev_index

        if index != prev_index:
            pba = power_bank_array[index]
            idx = 0
            for power in pba:
                self.set_power_bank(power_bank[idx], power)
                idx += 1
        return index

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)
