#! /usr/bin/python
# Copyright 2019, 2020 Cumulus Networks, Inc.

#############################################################################
# Platform and model specific poe subclass, inherits from the base poe class,
# and provides the followings:
# - loads the chip-specific poe implementation
#

try:
    import exceptions
    import os
    import termios
    import time
    import select
    import errno
    import fcntl
    import struct
    import serial
    import poe_base
    import poe_pd69200BT as poe_lib
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def poe():
    return PoeN3248PXE()

class PoeN3248PXE(poe_lib.PoePD69200BT):
    ''' POE UART settings are:
          port             - /dev/ttyS1
          baud rate        - 19200 bps
          stop bits        - 1
          parity           - none
          data bits        - 8
          flow control     - none
          interpacket gap  - > 20ms
          timeout          - 50ms
          byte gap         - < 2ms
    '''
    def __init__(self):
        poe_lib.PoePD69200BT.__init__(self)
        self.log_debug('poe_n3248pxe::init()')
        self._fd = None

    def poe_enable(self):
        self._enable_poe_in_cpld()

    def handle_warm_cold_boot(self):
        port_map = {
            0: (9, 8), 1: (11, 10), 2: (13, 12), 3: (15, 14),
            4: (1, 0), 5: (3, 2), 6: (5, 4), 7: (7, 6),
            8: (25, 24), 9: (27, 26), 10: (29, 28), 11: (31, 30),
            12: (17, 16), 13: (19, 18), 14: (21, 20), 15: (23, 22),
            16: (41, 40), 17: (43, 42), 18: (45, 44), 19: (47, 46),
            20: (33, 32), 21: (35, 34), 22: (37, 36), 23: (39, 38),
            24: (57, 56), 25: (59, 58), 26: (61, 60), 27: (63, 62),
            28: (49, 48), 29: (51, 50), 30: (53, 52), 31: (55, 54),
            32: (73, 72), 33: (75, 74), 34: (77, 76), 35: (79, 78),
            36: (65, 64), 37: (67, 66), 38: (69, 68), 39: (71, 70),
            40: (89, 88), 41: (91, 90), 42: (93, 92), 43: (95, 94),
            44: (81, 80), 45: (83, 82), 46: (85, 84), 47: (87, 86)
        }
        for port_id, port_phys in port_map.items():
            self.set_port_temp_matrix(port_id, port_phys[0], port_phys[1])
        self.set_active_matrix()
        self.set_individual_mask(0x00, 0)
        #self.set_pm_settings(range(48))
        self.set_port_enable_poe(range(48))

        ''' set temp thresholds
              140C => 0x8c
        '''
        for device in range(12):
            self.set_device_temp_threshold(device, 0x8c)

        self._persist_settings()

        ''' this will apply the saved configuration
        '''
        poe_lib.poe_pd69100.PoePD69100.handle_warm_cold_boot(self)

    def handle_poed_start(self):
        poe_lib.poe_pd69100.PoePD69100.handle_poed_start(self)
        '''
            enable POE in the CPLD
        '''
        self.enable_poe_in_cpld()

    @property
    def fd(self):
        if not self._fd:
            self._fd = serial.Serial('/dev/ttyS1', baudrate=19200, bytesize=8, parity='N', stopbits=1, timeout=1)
        return self._fd

    def enable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/1-0032/poe_enable', 'w').write('1')

    def disable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/1-0032/poe_enable', 'w').write('0')

    def poe_write(self, data):
        self.log_debug('poe_dell_n3248Ppxe::write()')
        return self.fd.write(struct.pack('B' * len(data), *data))

    def poe_write_unpacked(self, data):
        self.log_debug('poe_dell_n3248pxe::write_unpacked()')
        return self.fd.write(data)

    def poe_read(self, size=1):
        self.log_debug('poe_dell_n3248pxe::read()')
        read_data = self.fd.read(size)
        return struct.unpack('B' * len(read_data), read_data)

    def poe_read_unpacked(self, size=1):
        self.log_debug('poe_dell_n3248pxe::read_unpacked()')
        read_data = self.fd.read(size)
        return read_data

    def get_poe_params(self):
        ports = {}
        for port_id in range(48):
            ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                    True, poe_base.MAX_POEPLUS_POWER, True, poe_base.MAX_UPOE_POWER)
        sensors = {}
        for sensor_id in range(12):
            sensors[sensor_id] = poe_base.PoeSensorParams('temp%u' % (sensor_id + 1),
                                                          sensor_id, 120000, 150000)
        return poe_base.PoeParams(True, 1, ports, sensors)

    def adjust_system_poe_power(self, prev_index=-1):
        power_bank_array = [
        #   PB12  PB13  PB14
            (0,    0,     0),    # 0: no power supplies, never used
            (0,    0,  1056),    # 1: PSU1 1600
            (0,   1056,   0),    # 2: PSU2 1600
            (2496, 0,     0),    # 3: PSU1 1600,  PSU2 1600
        ]

        power_bank = (12, 13, 14)
        psu1_val = 0
        psu2_val = 0
        try:
            if int(open('/sys/bus/i2c/devices/1-0032/psu_pwr1_all_ok').read()) == 1:
                psu1_val = 1
                inp = open('/sys/bus/i2c/devices/30-0056/eeprom').read()[:5]
                if inp == '1600W':
                    psu1_val = 1
        except:
            pass

        try:
            if int(open('/sys/bus/i2c/devices/1-0032/psu_pwr2_all_ok').read()) == 1:
                inp = open('/sys/bus/i2c/devices/31-0056/eeprom').read()[:5]
                psu2_val = 1
                if inp == '1600W':
                    psu2_val = 2
        except:
            pass

        index = psu2_val + psu1_val
        if index == 0:
            return prev_index

        if index != prev_index:
            pba = power_bank_array[index]
            idx = 0
            for power in pba:
                self.set_power_bank(power_bank[idx], power)
                idx += 1
        return index

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)
