#! /usr/bin/python
# Copyright 2016 Cumulus Networks, Inc.

#############################################################################
# Platform and model specific poe subclass, inherits from the base poe class,
# and provides the followings:
# - loads the chip-specific poe implementation
#

try:
    import exceptions
    import os
    import termios
    import time
    import select
    import errno
    import fcntl
    import struct
    import serial
    import poe_base
    import poe_pd69100
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def poe():
    return poe_dni3048()

class poe_dni3048(poe_pd69100.PoePD69100):
    ''' according to dni3048 spec. POE port settings are:
          port             - /dev/ttyS1
          baud rate        - 19200 bps
          stop bits        - 1
          parity           - none
          data bits        - 8
          flow control     - none
          interpacket gap  - > 20ms
          timeout          - 50ms
          byte gap         - < 2ms
    '''
    def __init__(self):
        poe_pd69100.PoePD69100.__init__(self)
        self.log_debug('poe_dni3048t::init()')
        self._fd = None

    def poe_enable(self):
        self._enable_poe_in_cpld()

    def handle_warm_cold_boot(self):

        '''
        set our individual mask configurations. According to the Microsemi
        release note for PD69200 v1.88, the following mask bits must be set
        as follows:
          mask 0x08 (PD69100_IND_MASK_LEGACY_CLASS_MISMATCH) set to 0
          mask 0x38 (PD69100_IND_MASK_CLASS0_EQUAL_AF) set to 1
          mask 0x3b (PD69100_IND_MASK_CLASS_ERROR_EQUAL_0) set to 0
          mask 0x01 (PD69100_IND_MASK_LEGACY_ENABLED) set to 0
        '''
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_LEGACY_CLASS_MISMATCH, 0)
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_CLASS0_EQUAL_AF, 1)
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_CLASS_ERROR_EQUAL_0, 0)
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_LEGACY_ENABLED, 0)
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_PSE_POWERING_PSE, 1)
        port_map = {
             0:( 7,  6),  1:( 5,  4),  2:( 3, 2),   3:( 1,  0),
             4:(15, 14),  5:(13, 12),  6:(11, 10),  7:( 9,  8),
             8:(23, 22),  9:(21, 20), 10:(19, 18), 11:(17, 16),
            12:(31, 30), 13:(29, 28), 14:(27, 26), 15:(25, 24),
            16:(39, 38), 17:(37, 36), 18:(35, 34), 19:(33, 32),
            20:(47, 46), 21:(45, 44), 22:(43, 42), 23:(41, 40),
            24:(55, 54), 25:(53, 52), 26:(51, 50), 27:(49, 48),
            28:(63, 62), 29:(61, 60), 30:(59, 58), 31:(57, 56),
            32:(71, 70), 33:(69, 68), 34:(67, 66), 35:(65, 64),
            36:(79, 78), 37:(77, 76), 38:(75, 74), 39:(73, 72),
            40:(87, 86), 41:(85, 84), 42:(83, 82), 43:(81, 80),
            44:(95, 94), 45:(93, 92), 46:(91, 90), 47:(89, 88)
        }
        for port_id, port_phys in port_map.items():
            self.set_port_temp_matrix(port_id, port_phys[0], port_phys[1])
        self.set_active_matrix()

        self.set_port_disable_poe(range(48))

        ''' Use the Set PoE Device Register command to change PD device
            minimum startup time from the default 324ms to 400ms to allow
            some misbehaving devices to be discovered.
            We are setting the value 0x64 to reg 0x99, bits[11:5].
        '''
        self._set_device_reg_bits(0x99, 0x64, 0xfe0, 5)

        ''' turn on L2 management
        '''
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_LAYER2_LLDP, 1)
        self.set_pm_settings(poe_pd69100.PD69100_PM_MODE_PM1_LLDP,
                             poe_pd69100.PD69100_PM_MODE_PM2_ICUT,
                             poe_pd69100.PD69100_PM_MODE_PM3_NONE)

        time.sleep(0.1)
        ''' enable 4 pair on all ports
        '''
        self.set_individual_mask(poe_pd69100.PD69100_IND_MASK_4PAIR_ENABLE, 1)
        self.set_enable_4pair_mode(range(48))
        time.sleep(0.1)
        self.set_disable_4pair_power(range(48))

        ''' set temp thresholds
              140C => 0x8c
        '''
        for device in range(8):
            self.set_device_temp_threshold(device, 0x8c)

        self._persist_settings()

        ''' this will apply the saved configuration
        '''
        poe_pd69100.PoePD69100.handle_warm_cold_boot(self)

    def handle_poed_start(self):
        poe_pd69100.PoePD69100.handle_poed_start(self)
        '''
            enable POE in the CPLD
        '''
        self.enable_poe_in_cpld()

    @property
    def fd(self):
        if not self._fd:
            self._fd = serial.Serial('/dev/ttyS1', baudrate=19200, bytesize=8, parity='N', stopbits=1, timeout=1)
        return self._fd

    def enable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/0-0028/poe_plus_power', 'w').write('1')

    def disable_poe_in_cpld(self):
        open('/sys/bus/i2c/devices/0-0028/poe_plus_power', 'w').write('0')

    def poe_write(self, data):
        self.log_debug('poe_dni3048t::write()')
        return self.fd.write(struct.pack('B' * len(data), *data))

    def poe_write_unpacked(self, data):
        self.log_debug('poe_dni3048t::write_unpacked()')
        return self.fd.write(data)

    def poe_read(self, size=1):
        self.log_debug('poe_dni3048t::read()')
        read_data = self.fd.read(size)
        return struct.unpack('B' * len(read_data), read_data)

    def poe_read_unpacked(self, size=1):
        self.log_debug('poe_dni3048t::read_unpacked()')
        read_data = self.fd.read(size)
        return read_data

    def get_poe_params(self):
        ports = {}
        for port_id in range(48):
            ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                    True, poe_base.MAX_POEPLUS_POWER, True, poe_base.MAX_UPOE_POWER)
        sensors = {}
        for sensor_id in range(8):
            sensors[sensor_id] = poe_base.PoeSensorParams('temp%u' % (sensor_id + 1),
                                                          sensor_id, 120000, 150000)
        return poe_base.PoeParams(True, 1, ports, sensors)

    def adjust_system_poe_power(self, prev_index=-1):
        '''
          PSU slots status  |  psu value  |       |  POE Power Bank Setting       |
          ------------------+------+------+-------+-------------------------------+-----------------
            PSU1  |   PSU2  | PSU1 | PSU2 | index |   PB12   |   PB13   |   PB14  | Available Power
          --------+---------+------+------+-------+----------+---------+----------------------------
           1100W  |     x   |   2  |   0  |   2   |   950W   |     0    |   950W  |       950W
             x    |  1100W  |   0  |   2  |   6   |   950W   |   950W   |     0   |       950W
           1100W  |  1100W  |   2  |   2  |   8   |  1900W   |   950W   |   950W  |      1900W
            950W  |   950W  |   1  |   1  |   4   |  1600W   |   800W   |   800W  |      1600W
           1100W  |   950W  |   2  |   1  |   5   |  1750W   |   800W   |   950W  |      1750W
            950W  |  1100W  |   1  |   2  |   7   |  1750W   |   950W   |   800W  |      1750W
            950W  |     x   |   1  |   0  |   1   |   800W   |     0    |   800W  |       800W
              x   |   950W  |   0  |   1  |   3   |   800W   |   800W   |     0   |       800W
         ---------+---------+------+------+-------+----------+----------+---------+------------------
        '''
        power_bank_array = [
        #   PB12  PB13  PB14
            (0,      0,   0),    # 0: no power supplies, never used
            (800,    0, 800),    # 1: PSU1 950
            (950,    0, 950),    # 2: PSU1 1100
            (800,  800,   0),    # 3: PSU2 950
            (1600, 800, 800),    # 4: PSU1 950,  PSU2 950
            (1750, 800, 950),    # 5: PSU1 1100, PSU2 950
            (950,  950,   0),    # 6: PSU2 1100
            (1550, 950, 550),    # 7: PSU1 950, PSU2 1100
            (1900, 950, 950),    # 8: PSU1 1100, PSU2 1100
        ]

        power_bank = (12, 13, 14)
        psu1_val = 0
        psu2_val = 0
        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr1_all_ok').read()) == 1:
                psu1_val = 1
                inp = open('/sys/bus/i2c/devices/5-0052/eeprom').read()[:5]
                if inp == '1100W':
                    psu1_val = 2
        except:
            pass

        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr2_all_ok').read()) == 1:
                inp = open('/sys/bus/i2c/devices/6-0052/eeprom').read()[:5]
                psu2_val = 1
                if inp == '1100W':
                    psu2_val = 2
        except:
            pass

        index = psu2_val * 3 + psu1_val
        if index == 0:
            return prev_index

        if index != prev_index:
            pba = power_bank_array[index]
            idx = 0
            for power in pba:
                self.set_power_bank(power_bank[idx], power)
                idx += 1
        return index

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)
