#! /usr/bin/python
# Copyright 2015 Cumulus Networks Inc.  all rights reserved

#############################################################################
# Platform and model specific poe subclass, inherits from the base poe class,
# and provides the followings:
# - loads the chip-specific poe implementation
#

try:
    import exceptions
    import os
    import termios
    import time
    import select
    import errno
    import fcntl
    import struct
    import serial
    import poe_base
    import poe_pd69100
except ImportError, e:
    raise ImportError (str(e) + "- required module not found")

def poe():
    return poe_dni3448()

class poe_dni3448(poe_pd69100.PoePD69100):
    ''' according to dni3448 spec. POE port settings are:
          port             - /dev/ttyS1
          baud rate        - 19200 bps
          stop bits        - 1
          parity           - none
          data bits        - 8
          flow control     - none
          interpacket gap  - > 20ms
          timeout          - 50ms
          byte gap         - < 2ms
    '''
    def __init__(self):

        poe_pd69100.PoePD69100.__init__(self)
        self.log_debug('poe_dni3448t::init()')
        self._fd = None


    @property
    def fd(self):
        if not self._fd:
            self._fd = serial.Serial('/dev/ttyS1', baudrate=19200, bytesize=8, parity='N', stopbits=1, timeout=1)
        return self._fd

    def poe_system_init(self):
        '''
            enable POE in the CPLD
        '''
        open('/sys/bus/i2c/devices/0-0028/poe_plus_power', 'w').write('1')
        poe_pd69100.PoePD69100.__init__(self)

    def poe_write(self, data):
        self.log_debug('poe_dni3448t::write()')
        return self.fd.write(struct.pack('B' * len(data), *data))

    def poe_read(self, size=1):
        self.log_debug('poe_dni3448t::read()')
        read_data = self.fd.read(size)
        return struct.unpack('B' * len(read_data), read_data)

    def get_poe_params(self):
        '''
            UPOE is supported on ports 1-12
        '''
        ports = {}
        max_power = poe_base.MAX_POEPLUS_POWER
        upoe_supported = False
        for port_id in range(48):
            if port_id == 12:
                upoe_supported = False
                max_power = poe_base.MAX_POEPLUS_POWER
            ports[port_id] = poe_base.PoePortParams(port_id, 'swp%u' % (port_id + 1),
                                                    True, poe_base.MAX_POEPLUS_POWER, upoe_supported, max_power)
        return poe_base.PoeParams(True, 5, ports)

    def adjust_system_poe_power(self, prev_index=-1):
        ''' according to the dni3448 spec, we get

          PSU slots status  |   POE Power Bank Setting      |
          ------------------+-------------------------------+------------------
          PSU2    |   PSU1  |   PB12   |   PB13   |   PB14  | Available Power
          --------+---------+----------+----------+---------+------------------
           1100W  |     x   |      0   |   900W   |     0   |       900W
             x    |  1100W  |      0   |     0    |   900W  |       900W
           1100W  |  1100W  |   1800W  |   900W   |   900W  |      1800W
            715W  |   715W  |   1500W  |   550W   |   550W  |      1200W
           1100W  |   715W  |   1500W  |   900W   |   550W  |      1500W
            715W  |  1100W  |   1500W  |   550W   |   900W  |      1500W
            715W  |     x   |      0   |   550W   |     0   |       550W
              x   |   715W  |      0   |     0    |   550W  |       550W
         ---------+---------+----------+----------+---------+-------------------

        we set up a look-up table to get our values for pb12, pb13 and pb14. If we
        set things up exactly as indicated in the dni3448 spec, we will end up
        disconnecting devices when we go from one power supply to two (because we have
        set PB12 to zero in the case of a single PS and that gets immediately selected
        when a new PS is inserted, resulting in zero power). To account for this,
        we will set PB12 to be equal to the single supply's setting. This way we
        will not lose power. The power will be properly adjusted up when the new supply
        is discovered by poed.

              x         x                                                    0, 0
              x   |   715W  |    550W  |     0    |   550W  |       550W     0, 1
              x   |  1100W  |    900W  |     0    |   900W  |       900W     0, 2
            715W  |     x   |    550W  |   550W   |     0   |       550W     1, 0
            715W  |   715W  |   1200W  |   550W   |   550W  |      1200W     1, 1
            715W  |  1100W  |   1500W  |   550W   |   900W  |      1500W     1, 2
           1100W  |     x   |    900W  |   900W   |     0   |       900W     2, 0
           1100W  |   715W  |   1500W  |   900W   |   550W  |      1500W     2, 1
           1100W  |  1100W  |   1800W  |   900W   |   900W  |      1800W     2, 2
        '''

        self.get_power_bank_settings()
        power_bank_array = [
            (0,    0,   0),    # no power supplies, never used
            (900,  0,   900),
            (900,  0,   900),
            (550,  550, 0),
            (1200, 550, 550),
            (1500, 550, 900),
            (900,  900, 0),
            (1500, 900, 550),
            (1800, 900, 900),
        ]

        power_bank = (12, 13, 14)
        psu1_val = 0
        psu2_val = 0
        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr1_all_ok').read()) == 1:
                inp = open('/sys/bus/i2c/devices/5-0052/eeprom').read()[:5]
                psu1_val = 1
                if inp == '1100W':
                    psu1_val = 2
        except:
            pass

        try:
            if int(open('/sys/bus/i2c/devices/0-0028/psu_pwr2_all_ok').read()) == 1:
                inp = open('/sys/bus/i2c/devices/6-0052/eeprom').read()[:5]
                psu2_val = 1
                if inp == '1100W':
                    psu2_val = 2
        except:
            pass

        index = psu2_val * 3 + psu1_val
        if index == 0:
            return prev_index

        if index != prev_index:
            pba = power_bank_array[index]
            idx = 0
            for power in pba:
                self.set_power_bank(power_bank[idx], power)
                idx += 1
        return index

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)
